/*  commandir_set  Version 0.1 (in Utils 0.3)
    0.1 Initial Release: June 16th, 2011
 */

#include "cmdirutils_version.h"
#include "libcmdir.h"

extern unsigned int tx_bitmask;
extern char * send_filename;
extern struct commandir_device * first_commandir_device;


void setCommandIR_Order(usb_dev_handle * c, int order_num);
void setCommandIR_Name(usb_dev_handle * c, unsigned char * name);
void setCommandIR_Group(usb_dev_handle * c, unsigned char * name);
void setCommandIR_Mode(usb_dev_handle * c, int mode);
unsigned char * getCommandIR_pdata(usb_dev_handle * c);
unsigned char * getCommandIR_pdataRaw(usb_dev_handle * c);
//void hardware_setorder(void);
void setCommandIR_pdata(usb_dev_handle * c, unsigned char * pdata);
void pdataDisplay(unsigned char * pdata);
void graceful_close(void);
void insert_into_tx_order(int this_pcd_value, struct commandir_device * pcd);
void hardware_setorder_by_pdata_order(void);
struct commandir_device * getCommandIR_By_Name(unsigned char * n);

extern struct commandir_tx_order * ordered_commandir_devices;
extern unsigned char displayOnly;

unsigned char standalone = 1;


int main(int argc, char **argv)
{
  char * tx_char = NULL;
  int x, emitter_num;
  unsigned int msDelay = 200;
  struct commandir_device * pcd;
  struct commandir_tx_order * ptx;
  int busNum;
  int devId;
  usb_dev_handle * cmdirUpgrade;  // a udev device
  int order_num = 1;
  int y;

  
  if(argc == 1)
  {
    printf("commandir_set %s (For CommandIR III Firmware >= 3.42)\n", utils_version);
    printf(" Set CommandIR orders, groups, device names, and mini/expansion mode\n\n");
    printf("commandir_set -l\n"); // [g]
    printf("commandir_set -p\n"); // [g GROUP_NAME]
    printf("commandir_set -n NEW_NAME [BUS_NUM] [DEV_NUM]\n");
//    printf("commandir_set -g GROUP_NAME [COMMANDIR_NAME_1 [COMMANDIR_NAME_2] ...]\n");
    printf("commandir_set -r OLD_NAME NEW_NAME\n");
    printf("commandir_set -s COMMANDIR_NAME_1 [COMMANDIR_NAME_2] [COMMANDIR_NAME_3] ...\n");
    printf("commandir_set -e [-s COMMANDIR_NAME_1 [COMMANDIR_NAME_2] ...]\n");  // [-g GROUP_NAME] | 
    printf("commandir_set -m [-s COMMANDIR_NAME_1 [COMMANDIR_NAME_2] ...]\n");  // [-g GROUP_NAME] | 
    printf("commandir_set -d [-s COMMANDIR_NAME_1 [COMMANDIR_NAME_2] ...]\n");  // [-g GROUP_NAME] | 

    printf("\n");
    
    printf(" -l\tList the order and mode of attached CommandIRs\n");
    printf(" -p\tPersist the CommandIR order by USB connection order (LIRC's default)\n");
    printf(" -n\tName/rename a CommandIR by USB bus and device numbers\n");
//    printf(" -g\tSet the group name for specific or all (default) CommandIRs\n");
    printf(" -r\tRename a CommandIR\n");
    printf(" -s\tSet the order by specifying CommandIR names\n");
    printf(" -e\tSet the CommandIR(s) to operate in 'Expanded' mode (expansion required)\n");
    printf(" -m\tSet the CommandIR(s) to operate in 'Mini' mode\n");
    printf(" -d\tDelete order overrides. CommandIRs will order automatically by USB ids\n");

    // Only the GROUP FUNCTIONS LEFT.
//    printf(" -pg\tPersist CommandIR order by USB connection order for a particular group\n");
//    printf(" -lg\tList the current order of attached CommandIRs in their groups\n");
    return 0;
  }

  set_all_bitmask(0xff);
  
  for(x=1; x<argc; x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'e': // Put devices in expanded mode
        case 'm': // Put devices in mini mode
          // did they further specify -g or -s? 
          if((argc > x+1) && (argv[x+1][0] == '-')) {
            switch(argv[x+1][1]) {
              case 'g': // search for the items in this GROUP, and set their mode
                printf("Assigning by group not implemented yet\n");              
                break;
              case 's': // search for items with this NAME, and set their mode
                if(argc > x+1) {
                  hardware_scan();
                  for(y=x+2; y<argc; y++) {
                    pcd = getCommandIR_By_Name(argv[y]);
                    if(pcd) {
                      switch(argv[x][1]) {
                        case 'e':
                          printf("Setting CommandIR %s to operate in 'expansion' mode\n", argv[y]);
                          setCommandIR_Mode(pcd->cmdir_udev, 1);
                          break;
                        case 'm':
                          printf("Setting CommandIR %s to operate in 'mini' mode\n", argv[y]);
                          setCommandIR_Mode(pcd->cmdir_udev, 0);
                          break;
                      }
                    }
                  }
                }
                exit(0);  // otherwise, we'll pick up the -s switch
            }
          } else {
            // Put them ALL in this mode.
            hardware_scan();
            for(pcd = first_commandir_device; pcd; pcd = pcd->next_commandir_device) {
              // Set this one's group, if it's firmware supports pdata
              switch(argv[x][1]) {
                case 'e':
                  printf("Setting CommandIR to operate in 'expansion' mode\n");
                  setCommandIR_Mode(pcd->cmdir_udev, 1);
                  break;
                case 'm':
                  printf("Setting CommandIR to operate in 'mini' mode\n");
                  setCommandIR_Mode(pcd->cmdir_udev, 0);
                  break;
              }
            }
          }
          break;
        case 'd':
          // did they further specify -g or -s? 
          if(argc>x+1) {
            if(argv[x+1][0] == '-') {
              switch(argv[x+1][1]) {
                case 'g': // search for the items in this GROUP, and set their mode
                  printf("Deleting by group not implemented yet\n");              
                  break;
                case 's': // search for items with this NAME, and set their mode
                  if(argc > x+1) {
                    hardware_scan();
                    for(y=x+2; y<argc; y++) {
                      pcd = getCommandIR_By_Name(argv[y]);
                      if(pcd) {
                        printf("Clearing specific ordering information for CommandIR: %s\n", argv[y]);
                        setCommandIR_Order(pcd->cmdir_udev, 0);
                      } else {
                        printf("CommandIR %s not found.\n", argv[y]);
                      }
                    }
                  }
                  exit(0);  // otherwise, we'll pick up the -s switch
              }
            }
          } else {
            // Put them ALL in this mode.
            hardware_scan();
            for(pcd = first_commandir_device; pcd; pcd = pcd->next_commandir_device) {
              // Set this one's group, if it's firmware supports pdata
              setCommandIR_Order(pcd->cmdir_udev, 0);
            }
          }
          break;
        case 's':
          // Set order by specifying CommandIRs
          hardware_scan();
          if(argc >= 3) {
            // SPECIFIC CommandIRs
            order_num = 1;
            for(y=x+1; y<argc; y++) {
              pcd = getCommandIR_By_Name(argv[y]);
              if(pcd) {
                printf("Setting %s (at %d) to %d\n", argv[y], y, order_num);
                setCommandIR_Order(pcd->cmdir_udev, order_num++);
              } else {
                printf("Not found: %s\n", argv[y]);
              }
            }
          } else {
            printf("ERROR: Specify the CommandIR names to order this way\n");
          }          
          break;
        case 'r':
          hardware_scan();
          if(argc != 4) {
            printf("Incorrect parameters specified for renaming CommandIR\n");
            break;
          }
          pcd = getCommandIR_By_Name(argv[2]);
          if(pcd) 
            setCommandIR_Name(pcd->cmdir_udev, argv[3]);
          break;
        case 'g':
          // Set GROUP of either the specified CommandIRs, or ALL the attached CommandIRs.
          hardware_scan();
          if(argc > 3) {
            // SPECIFIC CommandIRs
            for(y=3; y<argc; y++) {
              // same?
              pcd = getCommandIR_By_Name(argv[2]);
              if(pcd)
                setCommandIR_Group(pcd->cmdir_udev, argv[3]);
            }
          } else {
            // ALL CommandIRs receive this group name
            for(pcd = first_commandir_device; pcd; pcd = pcd->next_commandir_device) {
              // Set this one's group, if it's firmware supports pdata
              setCommandIR_Group(pcd->cmdir_udev, argv[2]);
            }
          }
          
          break;
        case 'l':
          // Show the persistent vars of the attached CommandIRs - sort by order?
          y = hardware_scan();
          hardware_setorder_by_pdata_order();
          // Now, display the info in order
//          exit(0);
          order_num = 1;
          for(ptx = ordered_commandir_devices; ptx; ptx = ptx->next) {
            printf("CommandIR #%d (On USB %d:%d):\n", 
              order_num++,
              ptx->the_commandir_device->busnum,
              ptx->the_commandir_device->devnum);
            pdataDisplay(getCommandIR_pdataRaw(ptx->the_commandir_device->cmdir_udev));
          }
          graceful_close();
          return 0;
        case 'n': // Set a CommandIR name; get the CommandIR at the specified ID, reload it's persistent
          if(argc < 5) {
            printf("Not enough parameters for -n\n");
            return -1;
          }
          busNum = atoi(&argv[x+2][0]);
          devId = atoi(&argv[x+3][0]);
          printf("Searching for CommandIR at %d:%d\n", busNum, devId);
          cmdirUpgrade = hardware_scan_for(busNum, devId);
          if(cmdirUpgrade == NULL) {
            printf("No CommandIR found at USB %d:%d\n", busNum, devId);
            return -1;
          }
          setCommandIR_Name(cmdirUpgrade, argv[x+1]);
          graceful_close();
          return 0;
        case 'p': // Persist by USB order
          // Sort by bus-dev - we've done this in the LIRC driver
          hardware_scan();
//          create_pcd();       // setup the first_commandir chain - which is something we can order.
          hardware_setorder();
          // Now, write the order to the device
          order_num = 0;
          for(ptx = ordered_commandir_devices; ptx; ptx = ptx->next) {
            order_num++;
            setCommandIR_Order(ptx->the_commandir_device->cmdir_udev, order_num);
          }
          graceful_close();
          return 0;
      }
    }
  }
  return 0;
}

void graceful_close() {
  // Shutdown / unclaim the devices
}


